package io.sixminutes.breakingnews

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.util.Log
import com.applovin.mediation.MaxAd
import com.applovin.mediation.MaxAdListener
import com.applovin.mediation.MaxError
import com.applovin.mediation.ads.MaxInterstitialAd
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import java.util.concurrent.TimeUnit
import kotlin.math.pow

class InterstitialActivity : Activity(), MaxAdListener {
    private lateinit var interstitialAd: MaxInterstitialAd
    private var retryAttempt = 0
    private val maxRetryAttempts = 6 // 最大重试次数
    private var adId: String? = null // 广告 ID
    private val scope = CoroutineScope(Dispatchers.Main) // 创建一个 CoroutineScope
    private val TAG = "breakingNews"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        ActivityManager.addActivity(this)

        // 获取从 MainActivity 传递过来的广告 ID
        adId = intent.getStringExtra("AD_ID")
        if (adId != null) {
            createInterstitialAd(adId!!)
        } else {
            Log.e(TAG, "Ad ID is null. Cannot load ad.")
            setAdStatusResult("Ad ID is null. Cannot load ad.")
            finish()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        ActivityManager.removeActivity(this)
    }

    private fun createInterstitialAd(adId: String) {
        interstitialAd = MaxInterstitialAd(adId, this)
        interstitialAd.setListener(this)
        interstitialAd.loadAd()
    }

    // MAX Ad Listener
    override fun onAdLoaded(maxAd: MaxAd) {
        Log.d(TAG, "Ad loaded successfully: ${maxAd.adUnitId}")
        retryAttempt = 0

        // 显示广告
        if (interstitialAd.isReady) {
            interstitialAd.showAd(this)
        }

        // 返回广告加载成功状态
        setAdStatusResult("Ad Loaded: ${maxAd.adUnitId}")
    }

    override fun onAdLoadFailed(adUnitId: String, error: MaxError) {
        Log.d(TAG, "Ad load failed: $adUnitId, error code: ${error.code}, message: ${error.message}")

        if (retryAttempt < maxRetryAttempts) {
            retryAttempt++
            val delayMillis = TimeUnit.SECONDS.toMillis(2.0.pow(retryAttempt.toDouble()).toLong())
            scope.launch {
                delay(delayMillis)
                interstitialAd.loadAd()
            }
        } else {
            Log.d(TAG, "Maximum retry attempts reached. No more retries.")
            // 返回广告加载失败状态
            setAdStatusResult("Ad Load Failed: $adUnitId")
        }
    }

    override fun onAdDisplayFailed(ad: MaxAd, error: MaxError) {
        Log.d(TAG, "Ad display failed: ${ad.adUnitId}, error code: ${error.code}, message: ${error.message}")
        interstitialAd.loadAd()
    }

    override fun onAdDisplayed(maxAd: MaxAd) {
        Log.d(TAG, "Ad displayed: ${maxAd.adUnitId}")
    }

    override fun onAdClicked(maxAd: MaxAd) {
        Log.d(TAG, "Ad clicked: ${maxAd.adUnitId}")
    }

    override fun onAdHidden(maxAd: MaxAd) {
        Log.d(TAG, "Ad hidden: ${maxAd.adUnitId}")
        setAdStatusResult("Ad Hidden: ${maxAd.adUnitId}")
        finish() // 广告关闭后结束当前 Activity
    }

    // 设置广告状态结果
    private fun setAdStatusResult(status: String) {
        val resultIntent = Intent().apply {
            putExtra("AD_STATUS", status)
        }
        setResult(RESULT_OK, resultIntent)
    }
}